import { Component, ViewChild, Output, EventEmitter, AfterViewChecked, Inject, forwardRef } from "@angular/core";

import { FixedVarResultsComponent } from "../../components/fixedvar-results/fixedvar-results.component";
import { SectionResultsComponent } from "../../components/section-results/section-results.component";
import { RemousResultsComponent } from "../../components/remous-results/remous-results.component";
import { PabResultsComponent } from "../../components/pab-results/pab-results.component";
import { MacrorugoCompoundResultsComponent } from "../macrorugo-compound-results/macrorugo-compound-results.component";
import { FormulaireDefinition } from "../../formulaire/definition/form-definition";
import { JetResultsComponent } from "../jet-results/jet-results.component";
import { GenericCalculatorComponent } from "../generic-calculator/calculator.component";

@Component({
    selector: "calc-results",
    templateUrl: "./calculator-results.component.html",
})
export class CalculatorResultsComponent implements AfterViewChecked {

    private _formulaire: FormulaireDefinition;

    /**
     * composant d'affichage des résultats de paramètres fixés/variables
     */
    @ViewChild(FixedVarResultsComponent, { static: true })
    private fixedVarResultsComponent: FixedVarResultsComponent;

    /**
     * composant d'affichage des résultats des sections paramétrées
     */
    @ViewChild(SectionResultsComponent, { static: true })
    private sectionResultsComponent: SectionResultsComponent;

    /**
     * composant d'affichage des résultats des courbes de remous
     */
    @ViewChild(RemousResultsComponent, { static: true })
    private remousResultsComponent: RemousResultsComponent;

    /**
     * composant d'affichage des résultats des passes à bassins
     */
    @ViewChild(PabResultsComponent, { static: true })
    private pabResultsComponent: PabResultsComponent;

    /**
     * composant d'affichage des résultats des passes à macrorugosités complexes
     */
    @ViewChild(MacrorugoCompoundResultsComponent, { static: true })
    private mrcResultsComponent: MacrorugoCompoundResultsComponent;

    /**
     * composant d'affichage des résultats des impacts de jet
     */
    @ViewChild(JetResultsComponent, { static: true })
    private jetResultsComponent: JetResultsComponent;

    /**
     * événement émis à la fin du dessin de la vue
     */
    @Output()
    private afterViewChecked = new EventEmitter();

    public constructor(
        @Inject(forwardRef(() => GenericCalculatorComponent)) private calculatorComponent: GenericCalculatorComponent
    ) { }

    public set formulaire(f: FormulaireDefinition) {
        this._formulaire = f;
        if (this._formulaire === undefined) {
            this.fixedVarResultsComponent.results = undefined;
            this.sectionResultsComponent.results = undefined;
            this.remousResultsComponent.results = undefined;
            this.pabResultsComponent.results = undefined;
            this.mrcResultsComponent.results = undefined;
            this.jetResultsComponent.results = undefined;
        } else {
            this.sectionResultsComponent.results = f.results;
            this.remousResultsComponent.results = f.results;
            this.pabResultsComponent.results = f.results;
            this.mrcResultsComponent.results = f.results;
            // FixedVar and Jet are mutually incompatible (the 2nd extend the 1st)
            if (this.isJet) {
                this.jetResultsComponent.results = f.results;
                this.fixedVarResultsComponent.results = undefined;
            } else {
                this.fixedVarResultsComponent.results = f.results;
                this.jetResultsComponent.results = undefined;
            }
        }
    }

    public updateView() {
        this.fixedVarResultsComponent.updateView();
        this.sectionResultsComponent.updateView();
        this.remousResultsComponent.updateView();
        this.pabResultsComponent.updateView();
        this.mrcResultsComponent.updateView();
        this.jetResultsComponent.updateView();
    }

    public ngAfterViewChecked() {
        this.afterViewChecked.emit();
    }

    public get isJet() {
        return this.calculatorComponent.isJet;
    }
}
