import { IObservable, ParamDefinition, Nub } from "jalhyd";

import { SelectFieldNub } from "../elements/select-field-nub";
import { SelectFieldParameter } from "../elements/select-field-parameter";
import { NgParameter } from "../elements/ngparam";
import { FormulaireFixedVar } from "./form-fixedvar";
import { SelectField } from "../elements/select-field";
import { FieldSet } from "../elements/fieldset";

/**
 * Formulaire pour les Solveurs
 */
export class FormulaireSolveur extends FormulaireFixedVar {

    /** id of select configuring target Nub */
    private _targetNubSelectId: string;

    /** id of select configuring targetted result */
    private _targettedResultSelectId: string;

    /** id of select configuring searched param */
    private _searchedParamSelectId: string;

    protected parseOptions(json: {}) {
        super.parseOptions(json);
        this._targetNubSelectId = this.getOption(json, "targetNubSelectId");
        this._targettedResultSelectId = this.getOption(json, "targettedResultSelectId");
        this._searchedParamSelectId = this.getOption(json, "searchedParamSelectId");
    }

    protected completeParse(json: {}, firstNotif: boolean = true) {
        super.completeParse(json);
        if (this._targetNubSelectId) {
            const sel = this.getFormulaireNodeById(this._targetNubSelectId);
            if (sel) {
                sel.addObserver(this);
                if (firstNotif) {
                    // force 1st observation
                    (sel as SelectFieldNub).notifySelectValueChanged();
                }
            }
        }
        if (this._targettedResultSelectId) {
            const sel = this.getFormulaireNodeById(this._targettedResultSelectId);
            if (sel) {
                sel.addObserver(this);
                if (firstNotif) {
                    // force 1st observation
                    (sel as SelectField).notifyValueChanged();
                }
            }
        }
        if (this._searchedParamSelectId) {
            const sel = this.getFormulaireNodeById(this._searchedParamSelectId);
            if (sel) {
                sel.addObserver(this);
                if (firstNotif) {
                    // force 1st observation
                    (sel as SelectFieldNub).notifySelectValueChanged();
                }
            }
        }

    }

    // interface Observer

    public update(sender: IObservable, data: any) {
        // copied from FormDefinition, to avoid calling super.update()
        if (sender instanceof Nub) {
            switch (data.action) {
                case "resultUpdated":
                    // forward Nub results update notification to FormCompute objects
                    this.reaffectResultComponents();
                    break;
            }
        }
        // copied from FormFixedVar, to avoid calling super.update()
        if (data.action === "propertyChange") {
            this.reset();
        }

        if (sender instanceof SelectFieldNub) {
            if (data.action === "select") {
                // update Solveur property: Nub to calculate
                try {
                    // if searchedParam is set to a value that won't be available anymore
                    // once nubToCalculate is updated, setPropValue throws an error, but
                    // nubToCalculate is updated anyway; here, just inhibit the error
                    this._currentNub.properties.setPropValue("nubToCalculate", data.value.value);
                } catch (e) { }
                // refresh targetted result selector
                const trSel = this.getFormulaireNodeById(this._targettedResultSelectId) as SelectField;
                if (trSel) {
                    (trSel.parent as FieldSet).updateFields();
                    // trick to re-set observers
                    this.completeParse({}, false);
                }
                // refresh parameters selector
                this.refreshParameterEntries();
            }
        } else if (sender instanceof SelectFieldParameter) {
            if (data.action === "select") {
                // update Solveur property: searched Parameter
                try {
                    const p: ParamDefinition = data.value.value;
                    this._currentNub.properties.setPropValue(
                        "searchedParameter",
                        p.nubUid + "/" + p.symbol
                    );
                } catch (e) { }
                // reflect changes in GUI
                const inputXinit = this.getFormulaireNodeById("Xinit") as NgParameter;
                inputXinit.notifyValueModified(this);
            }
        } else if (sender instanceof SelectField) {
            if (sender.id === "select_target_result") {
                // refresh parameters selector
                this.refreshParameterEntries();
            }
        }
    }

    /**
     * Re-populate searched parameter selector with fresh entries
     */
    private refreshParameterEntries() {
        const pSel = this.getFormulaireNodeById(this._searchedParamSelectId) as SelectFieldParameter;
        if (pSel) {
            pSel.updateEntries();
            // reflect changes in GUI
            const inputYtarget = this.getFormulaireNodeById("Ytarget") as NgParameter;
            inputYtarget.notifyValueModified(this);
        }
    }
}
